@@ -55,6 +55,33 @@
 #endif
 
 
+namespace Givaro
+{
+    inline void reconstructRational (Integer& a, Integer& b, const Integer& x, const Integer& m, const Integer& bound)
+    {
+        Rational rr(x,m,bound);
+        a = rr.nume();
+        b = rr.deno();
+    }
+    
+    inline void reconstructRational (Integer& a, Integer& b, const Integer& x, const Integer& m)
+    {
+        reconstructRational(a,b, x, m, Givaro::sqrt(m));
+    }
+    
+    inline bool reconstructRational (Integer& a, Integer& b,
+                                     const Integer& x, const Integer& m,
+                                     const Integer& a_bound, const Integer& b_bound)
+    {
+        Integer bound = x/b_bound;
+			// if (bound>a_bound) std::cerr << "a_bound: " << a_bound << ", x/b_bound: " << bound << std::endl;
+        
+        reconstructRational(a,b,x,m, (bound>a_bound?bound:a_bound));
+        
+        return b<= b_bound;
+    }
+    
+}
 
 
 namespace LinBox
@@ -64,6 +91,9 @@ namespace LinBox
 		return ( (m.bitsize()+7 )/8) ;
 	}
 
+
+    
+
 	/*! \brief Limited doc so far.
 	 * Used, for instance, after LiftingContainer.
 	 */
@@ -169,7 +199,7 @@ namespace LinBox
 		{
 			typename InVect1::const_iterator v1_p;
 			typename InVect2::const_iterator v2_p;
-			_r. init (d, 0);
+			_r. assign(d, _r.zero);
 			for (v1_p = v1. begin(), v2_p = v2. begin(); v1_p != v1. end(); ++ v1_p, ++ v2_p)
 				_r. axpyin (d, *v1_p, *v2_p);
 
@@ -209,25 +239,26 @@ namespace LinBox
 			int n   = (int)num. size();
 			int len = (int)_lcontainer. length();
 			Integer prime = _lcontainer.prime();//prime
-			LVector digits; //Store all p-adic digits
-			digits. resize (len); //reserve space for all digits
+			const BlasVector<Ring> zero(_r,(size_t)n);
+			LVector digits((size_t)len,zero); //Store all p-adic digits
+			// digits. resize ((size_t)len); //reserve space for all digits
 			Integer modulus; //store current modulus
 			Integer denbound; // store current bound for den
 			Integer numbound; //store current bound for num
 
-			_r. init (modulus, 1);
-			_r. init (denbound, 1);
-			_r. init (numbound, 1);
+			_r. assign (modulus, _r.one);
+			_r. assign (denbound, _r.one);
+			_r. assign (numbound, _r.one);
 			Integer c1, c2, c1_den, c1_num, c2_den, c2_num;
-			IVector r1(num.size()), r2(num.size());
-			_r. init (c1, 0); _r. init(c1_den, 1); _r. init (c1_num, 0);
-			_r. init (c2, 0); _r. init(c2_den, 1); _r. init (c2_num, 0);
+			IVector r1(_r,num.size()), r2(_r,num.size());
+			_r. assign(c1, _r.zero); _r. assign(c1_den, _r.one); _r. assign (c1_num, _r.zero);
+			_r. assign(c2, _r.zero); _r. assign(c2_den, _r.one); _r. assign (c2_num, _r.zero);
 
 			typename IVector::iterator r_p;
 			for (r_p = r1. begin(); r_p != r1. end(); ++ r_p)
-				_r. init (*r_p, rand());
+				_r. init (*r_p, int64_t(rand()));
 			for (r_p = r2. begin(); r_p != r2. end(); ++ r_p)
-				_r. init (*r_p, rand());
+				_r. init (*r_p, int64_t(rand()));
 
 			//std::cout << "Random vecotor1: " ;
 			//print (r1);
@@ -237,7 +268,7 @@ namespace LinBox
 			Integer pmodulus; //store previous modulus
 			Integer tmp; //temprary integer
 
-			_r. init (den, 1);
+			_r. assign (den, _r.one);
 			typename LiftingContainer::const_iterator iter = _lcontainer.begin();
 
 			Integer tmp_den, tmp_num, rem1, rem2;
@@ -259,12 +290,13 @@ namespace LinBox
 				IVector& dig = *digits_p;
 				++step; ++ digits_p;
 
-				dig. resize (n);
+				// dig. resize ((size_t)n);
 
 				// get next p-adic digit
 				bool nextResult = iter.next(dig);
 				if (!nextResult) {
-					std::cout << "ERROR in lifting container. Are you using <double> ring with large norm? (1)" << std::endl;
+				commentator().report() <<
+					"ERROR in lifting container. Are you using <double> ring with large norm? (1)" << std::endl;
 					return false;
 				}
 				//std::cout << "New digits:\n";
@@ -283,14 +315,14 @@ namespace LinBox
 				if ((step % _threshold) == 0) {
 
 					//std::cout << "Previous (Current) modulus: " << pmodulus << "( " << modulus << ")\n";
-					dot (tmp, r1, dig); _r. remin (tmp, prime); _r. axpyin (c1, tmp, pmodulus);
+					dot (tmp, r1, dig); _r. modin (tmp, prime); _r. axpyin (c1, tmp, pmodulus);
 					//std::cout << "r1 * digit: " << tmp << '\n';
-					dot (tmp, r2, dig); _r. remin (tmp, prime); _r. axpyin (c2, tmp, pmodulus);
+					dot (tmp, r2, dig); _r. modin (tmp, prime); _r. axpyin (c2, tmp, pmodulus);
 					//std::cout << "r2 * digit: " << tmp << '\n';
 					//std::cout << "c1, c2: " << c1 << ", " << c2 << "\n";
 
-					_r. mul (rem1, c1, c1_den); _r. subin (rem1, c1_num); _r. remin (rem1, modulus);
-					_r. mul (rem2, c2, c2_den); _r. subin (rem2, c2_num); _r. remin (rem2, modulus);
+					_r. mul (rem1, c1, c1_den); _r. subin (rem1, c1_num); _r. modin (rem1, modulus);
+					_r. mul (rem2, c2, c2_den); _r. subin (rem2, c2_num); _r. modin (rem2, modulus);
 
 					//Early termination condition is met.
 
@@ -300,21 +332,21 @@ namespace LinBox
 					}
 
 					if (!_r. isZero (rem1)) {
-						int status = (int)_r.reconstructRational(tmp_num, tmp_den, c1, modulus, numbound, denbound);
+						int status = (int)Givaro::reconstructRational(tmp_num, tmp_den, c1, modulus, numbound, denbound);
 						if(status) {
 							_r. assign (c1_den, tmp_den); _r. assign (c1_num, tmp_num);
 						}
 					}
 
 					if (!_r. isZero (rem2)) {
-						int  status =(int)  _r.reconstructRational(tmp_num, tmp_den, c2, modulus, numbound, denbound);
+						int  status =(int) Givaro::reconstructRational(tmp_num, tmp_den, c2, modulus, numbound, denbound);
 						if(status) {
 							_r. assign (c2_den, tmp_den); _r. assign (c2_num, tmp_num);
 						}
 					}
 				}
 			}
-			IVector res (n);
+			IVector res (_r,(size_t)n);
 			typename LVector::const_iterator digit_begin = digits. begin();
 			PolEval (res, digit_begin, (size_t)step, prime);
 			if(step < len) _r. lcm (den, c1_den, c2_den);
@@ -332,11 +364,11 @@ namespace LinBox
 			std::cout << "Start rational reconstruction:\n";
 			typename Vector::iterator num_p; typename IVector::iterator res_p;
 			Integer tmp_res, neg_res, abs_neg, l, g;
-			_r. init (den, 1);
+			_r. assign(den, _r.one);
 			int counter=0;
 			for (num_p = num. begin(), res_p = res. begin(); num_p != num. end(); ++ num_p, ++ res_p) {
 				_r. mul (tmp_res, *res_p, den);
-				_r. remin (tmp_res, modulus);
+				_r. modin (tmp_res, modulus);
 				_r. sub (neg_res, tmp_res, modulus);
 				_r. abs (abs_neg, neg_res);
 
@@ -345,9 +377,10 @@ namespace LinBox
 				else if (_r. compare(abs_neg, numbound) < 0)
 					_r. assign (*num_p, neg_res);
 				else {
-					int status= (int) _r. reconstructRational(tmp_num, tmp_den, *res_p, modulus, numbound, denbound);
+					int status= (int)Givaro::reconstructRational(tmp_num, tmp_den, *res_p, modulus, numbound, denbound);
 					if (!status) {
-						std::cout << "ERROR in reconstruction ? (1)\n" << std::endl;
+						commentator().report() 
+						<< "ERROR in reconstruction ? (1)\n" << std::endl;
 #ifdef DEBUG_RR
 						std::cout<<" try to reconstruct :\n";
 						//	std::cout<<"approximation: "<<*iter_approx<<std::endl;
@@ -400,7 +433,7 @@ namespace LinBox
 #endif
 			linbox_check(num.size() == (size_t)_lcontainer.size());
 
-			_r. init (den, 1);
+			_r. assign (den, _r.one);
 			Integer prime = _lcontainer.prime(); 		        // prime used for lifting
 			std::vector<size_t> accuracy(_lcontainer.size(), 0); 	// accuracy (in powers of p) of each answer so far
 			Vector digit(_lcontainer.size());  		        // to store next digit
@@ -408,9 +441,9 @@ namespace LinBox
 			Integer prev_modulus;       	                        // store previous modulus
 			Integer numbound, denbound;                             // current num/den bound for early termination
 			size_t numConfirmed;                                       // number of reconstructions which passed twice
-			_r.init(modulus, 0);
+			_r.assign(modulus, _r.zero);
 			std::vector<Integer> zz(_lcontainer.size(), modulus);   // stores each truncated p-adic approximation
-			_r.init(modulus, 1);
+			_r.assign(modulus, _r.one);
 
 			size_t len = _lcontainer.length();
 			/* should be ceil(log(2*numbound*denbound)/log(prime))
@@ -457,7 +490,7 @@ namespace LinBox
 					_r.convert(iD, _lcontainer.numbound());
 					_r.convert(iN, _lcontainer.denbound());
 					_r.convert(pPower, prime);
-					pPower = pow(pPower, len-1);
+					pPower = pow(pPower, uint64_t(len)-1);
 
 					tmp = pPower * iN;
 					tmp /= iD;
@@ -501,7 +534,8 @@ namespace LinBox
 				// get next p-adic digit
 				bool nextResult = iter.next(digit);
 				if (!nextResult) {
-					std::cout << "ERROR in lifting container. Are you using <double> ring with large norm? (2)" << std::endl;
+					commentator().report()
+					<< "ERROR in lifting container. Are you using <double> ring with large norm? (2)" << std::endl;
 					return false;
 				}
 #ifdef RSTIMING
@@ -566,7 +600,7 @@ namespace LinBox
 						Integer tmp_num, tmp_den;
 						_r. assign (tmp_den, den);
 						_r. mul (tmp_num, den, *zz_p);
-						_r. remin (tmp_num, modulus);
+						_r. modin (tmp_num, modulus);
 
 						// assign tmp_num = one of tmp_num and tmp_num - modulus with smallest absolute value.
 						Integer n_num;
@@ -595,7 +629,7 @@ namespace LinBox
 						justConfirming = false;
 						// if no answer yet (or last answer became invalid)
 						// try to reconstruct a rational number
-						tmp = _r.reconstructRational(*num_p, tmp_den, *zz_p, modulus, numbound, denbound);
+						tmp = Givaro::reconstructRational(*num_p, tmp_den, *zz_p, modulus, numbound, denbound);
 						// update 'accuracy' according to whether it worked or not
 						if (tmp) {
 							linbox_check (!_r.isZero(tmp_den));
@@ -629,7 +663,7 @@ namespace LinBox
 							// check if the rational number works for _zz_p mod _modulus
 							_r. mul (tmp_i, den, *zz_p);
 							_r. subin (tmp_i, *num_p);
-							_r. remin (tmp_i, modulus);
+							_r. modin (tmp_i, modulus);
 							if (_r.isZero (tmp_i)) {
 								*accuracy_p = i;
 								numConfirmed++;
@@ -637,7 +671,7 @@ namespace LinBox
 							else {
 								// previous result is fake, reconstruct new answer
 								Integer tmp_den;
-								tmp = _r.reconstructRational(*num_p, tmp_den, *zz_p, modulus, numbound, denbound);
+								tmp = Givaro::reconstructRational(*num_p, tmp_den, *zz_p, modulus, numbound, denbound);
 								if (tmp) {
 									linbox_check (!_r.isZero(den));
 									if (! _r. areEqual (tmp_den, den)) {
@@ -672,7 +706,7 @@ namespace LinBox
 
 
 			Integer g;
-			_r. init (g, 0);
+			_r. assign (g, _r.zero);
 			_r. gcdin (g, den);
 			for (num_p = num. begin(); num_p != num. end(); ++ num_p)
 				_r. gcdin (g, *num_p);
@@ -698,27 +732,25 @@ namespace LinBox
 
 			if (deg == 1){
 				for (size_t i=0;i<y.size();++i)
-					_r.assign(y[i],(*Pol)[i]);
+					_r.assign(y[(size_t)i],(*Pol)[(size_t)i]);
 			}
 			else{
 				size_t deg_low, deg_high;
 				deg_high = deg/2;
 				deg_low  = deg - deg_high;
-				Integer zero;
-				_r.init(zero,0);
-				Vector y1(y.size(),zero), y2(y.size(),zero);
+				Vector y1(_r,y.size(),_r.zero), y2(_r,y.size(),_r.zero);
 				Integer x1=x, x2=x;
 
 				PolEval(y1, Pol, deg_low, x1);
 
-				ConstIterator Pol_high= Pol+deg_low;
+				ConstIterator Pol_high= Pol+(ptrdiff_t)deg_low;
 				PolEval(y2, Pol_high, deg_high, x2);
 
 
 				for (size_t i=0;i< y.size();++i){
-					_r.assign(y[i],y1[i]);
-					_r.axpyin(y[i],x1,y2[i]);
-					//_r.axpy(y[i],x1,y2[i],y1[i]);
+					_r.assign(y[(size_t)i],y1[(size_t)i]);
+					_r.axpyin(y[(size_t)i],x1,y2[(size_t)i]);
+					//_r.axpy(y[(size_t)i],x1,y2[(size_t)i],y1[(size_t)i]);
 				}
 
 				_r.mul(x,x1,x2);
@@ -751,20 +783,18 @@ namespace LinBox
 			size_t size= _lcontainer.size();
 
 
-			Integer zero;
-			_r.init(zero,0);
-			Vector zero_digit(_lcontainer.size(),zero);
+			Vector zero_digit(_r,_lcontainer.size(),_r.zero);
 
 			// store approximation as a polynomial and evaluate by baby step giant step
 			std::vector<Vector>  digit_approximation(length,zero_digit);
 
 			// store real approximation
-			Vector real_approximation(size,zero);
+			Vector real_approximation(_r,size,_r.zero);
 
 
 			// store modulus (intially set to 1)
 			Integer modulus;
-			_r.init(modulus, 1);
+			_r.assign(modulus, _r.one);
 
 			// denominator upper bound
 			Integer denbound;
@@ -788,7 +818,7 @@ namespace LinBox
 #endif
 			// Compute all the approximation using liftingcontainer
 			typename LiftingContainer::const_iterator iter = _lcontainer.begin();
-			for (size_t i=0 ; iter != _lcontainer.end() && iter.next(digit_approximation[i]);++i) {
+			for (size_t i=0 ; iter != _lcontainer.end() && iter.next(digit_approximation[(size_t)i]);++i) {
 
 #ifdef LIFTING_PROGRESS
 				lifting_commentator().progress(i);
@@ -796,7 +826,7 @@ namespace LinBox
 #if 0
 				eval_horn.start();
 				for (size_t j=0;j<size;++j)
-					_r.axpyin(real_approximation[j],modulus, digit_approximation[i][j]);
+					_r.axpyin(real_approximation[j],modulus, digit_approximation[(size_t)i][j]);
 				eval_horn.stop();
 				eval_horner+=eval_horn;
 #endif
@@ -809,7 +839,8 @@ namespace LinBox
 
 			// problem occured during lifting
 			if (iter!= _lcontainer.end()){
-				std::cout << "ERROR in lifting container. Are you using <double> ring with large norm? (3)" << std::endl;
+				commentator().report()
+				<< "ERROR in lifting container. Are you using <double> ring with large norm? (3)" << std::endl;
 				return false;
 			}
 
@@ -845,7 +876,7 @@ namespace LinBox
 				for (int i= length -1; i>= skip+sqrt_length; --i)
 					for (size_t j=0;j<size;++j) {
 						_r.mulin(baby_approx[sqrt_length][j] , prime);
-						_r.addin(baby_approx[sqrt_length][j], digit_approximation[i][j]);
+						_r.addin(baby_approx[sqrt_length][j], digit_approximation[(size_t)i][j]);
 					}
 
 				LinBox::integer p_to_sqrt, p;
@@ -859,7 +890,7 @@ namespace LinBox
 				for (int i= sqrt_length; i>= 0; --i)
 					for (size_t j=0;j<size;j++) {
 						_r.mulin(real_approximation[j] , prime_to_sqrt );
-						_r.addin(real_approximation[j], baby_approx[i][j]);
+						_r.addin(real_approximation[j], baby_approx[(size_t)i][j]);
 					}
 			}
 			eval_bsgs.stop();
@@ -896,10 +927,11 @@ namespace LinBox
 				typename Vector::iterator   iter_d  = den_r.begin();
 
 				for (size_t i=0; iter_a != real_approximation.end(); ++iter_a, ++ iter_n, ++iter_d, ++i){
-					if (!_r.reconstructRational(*iter_n, *iter_d,
+					if (!Givaro::reconstructRational(*iter_n, *iter_d,
 								    *iter_a, modulus, numbound, denbound))
 					{
-						std::cout << "ERROR in reconstruction ?\n" << std::endl;
+						commentator().report()
+						<< "ERROR in reconstruction ?\n" << std::endl;
 					}
 				}
 				dumb_ratrecon.stop();
@@ -914,11 +946,11 @@ namespace LinBox
 			Timer ratrecon;
 			ratrecon.start();
 			Integer common_den, common_den_mod_prod, bound,two,tmp;
-			_r.init(common_den,1);
-			_r.init(common_den_mod_prod,1);
-			_r.init(two,2);
+			_r.assign(common_den,_r.one);
+			_r.assign(common_den_mod_prod,_r.one);
+			_r.init(two,int64_t(2));
 
-			Vector denominator(num.size());
+			Vector denominator(_r,num.size());
 
 			int counter=0;
 			typename Vector::iterator   iter_approx = real_approximation.begin();
@@ -932,23 +964,23 @@ namespace LinBox
 			for (size_t i=0; iter_approx != real_approximation.end(); ++iter_approx, ++ iter_num, ++iter_denom, ++i){
 				//_r.mulin( *iter_approx , common_den_mod_prod);
 				_r.mulin( *iter_approx , common_den);
-				_r.remin( *iter_approx , modulus);
+				_r.modin( *iter_approx , modulus);
 				_r. sub (neg_approx, *iter_approx, modulus);
 				_r. abs (abs_approx, neg_approx);
 
 				if ( _r.compare(*iter_approx, numbound) < 0){
 					_r.assign(*iter_num, *iter_approx);
-					_r.init(*iter_denom, 1);
+					_r.assign(*iter_denom, _r.one);
 				}
 				else if (_r.compare(abs_approx, numbound) <0){
 					_r.assign(*iter_num, neg_approx);
-					_r.init(*iter_denom, 1);
+					_r.assign(*iter_denom, _r.one);
 				}
 				else {
-					if  (!_r.reconstructRational(*iter_num, *iter_denom, *iter_approx, modulus, numbound, denbound))
+					if  (!Givaro::reconstructRational(*iter_num, *iter_denom, *iter_approx, modulus, numbound, denbound))
 					{
-						std::cout << "ERROR in reconstruction ? (3)\n" << std::endl;
 #ifdef DEBUG_RR
+						std::cout << "ERROR in reconstruction ? (3)\n" << std::endl;
 						std::cout<<" try to reconstruct :\n";
 						std::cout<<"approximation: "<<*iter_approx<<std::endl;
 						std::cout<<"modulus: "<<modulus<<std::endl;
@@ -975,10 +1007,10 @@ namespace LinBox
 							_r.assign(modulus,tmp);
 							_r.div(tmp,modulus,prime);
 						}
-						_r.rem(tmp , *iter_denom , modulus);
-						_r.remin(common_den_mod_prod , modulus);
+						_r.mod(tmp , *iter_denom , modulus);
+						_r.modin(common_den_mod_prod , modulus);
 						_r.mulin(common_den_mod_prod , tmp);
-						_r.remin(common_den_mod_prod , modulus);
+						_r.modin(common_den_mod_prod , modulus);
 					}
 #endif
 
@@ -986,17 +1018,17 @@ namespace LinBox
 
 			}
 
-			_r.init(tmp,1);
+			_r.assign(tmp,_r.one);
 			for (int i= idx_last_den ; i>=0;--i){
-				_r.mulin(num[i],tmp);
-				_r.mulin(tmp,denominator[i]);
+				_r.mulin(num[(size_t)i],tmp);
+				_r.mulin(tmp,denominator[(size_t)i]);
 			}
 
 
 #if 0
 			typename Vector1::reverse_iterator rev_iter_num   = num.rbegin();
 			typename Vector::reverse_iterator  rev_iter_denom = denominator.rbegin();
-			_r.init(tmp,1);
+			_r.assign(tmp,_r.one);
 			for (; rev_iter_num != num.rend(); ++rev_iter_num, ++rev_iter_denom){
 
 				_r.mulin(*rev_iter_num,tmp);
@@ -1034,7 +1066,7 @@ namespace LinBox
 
 			Integer init_den = den_app;
 			if (den > 0) lcm(init_den,den,den_app);
-			_r. init (den, 1);
+			_r. assign(den, _r.one);
 #ifdef DEBUG_RR
 			cout << "debug: den " << den;
 #endif
@@ -1042,9 +1074,9 @@ namespace LinBox
 			Vector digit(_lcontainer.size());                       // to store next digit
 			Integer modulus;                                        // store modulus (power of prime)
 			Integer prev_modulus;                                   // store previous modulus
-			_r.init(modulus, 0);
+			_r.assign(modulus, _r.zero);
 			std::vector<Integer> zz(_lcontainer.size(), modulus);   // stores each truncated p-adic approximation
-			_r.init(modulus, 1);
+			_r.assign(modulus, _r.one);
 
 			size_t len = _lcontainer.length(); // should be ceil(log(2*numbound*denbound)/log(prime))
 #ifdef DEBUG_RR
@@ -1078,7 +1110,8 @@ namespace LinBox
 				// get next p-adic digit
 				bool nextResult = iter.next(digit);
 				if (!nextResult) {
-					std::cout << "ERROR in lifting container. Are you using <double> ring with large norm? (ET)" << std::endl;
+					commentator().report()
+					<< "ERROR in lifting container. Are you using <double> ring with large norm? (ET)" << std::endl;
 					return false;
 				}
 #ifdef RSTIMING
@@ -1111,7 +1144,7 @@ namespace LinBox
 					terminated = true;
 					for ( zz_p = zz.begin(), num_p = num.begin(); zz_p != zz.end();  ++ zz_p, ++ num_p) {
 						Integer a = *num_p;
-						Integer bx= *zz_p; _r. mulin (bx, den); _r. remin(bx, modulus);
+						Integer bx= *zz_p; _r. mulin (bx, den); _r. modin(bx, modulus);
 						Integer _bx = bx-modulus;
 						if (!_r.areEqual(a,bx) && !_r.areEqual(a,_bx)) {
 							terminated = false;
@@ -1129,8 +1162,8 @@ namespace LinBox
 					Integer tmp_den=0;
 					Integer zz_p_den (*zz_p);
 					_r. mulin (zz_p_den,den);
-					_r. remin (zz_p_den,modulus);
-					bool tmp = RR.reconstructRational(*num_p, tmp_den, zz_p_den, modulus);
+					_r. modin (zz_p_den,modulus);
+					bool tmp = Givaro::reconstructRational(*num_p, tmp_den, zz_p_den, modulus);
 #ifdef RSTIMING
 					++counter;
 #endif
@@ -1158,9 +1191,9 @@ namespace LinBox
 					Integer tmp_den;
 					Integer zz_p_den (*zz_p);
 					_r. mulin (zz_p_den,den);
-					_r. remin (zz_p_den,modulus);
+					_r. modin (zz_p_den,modulus);
 
-					bool tmp = RR.reconstructRational(*num_p, tmp_den, zz_p_den, modulus, _lcontainer.numbound(), _lcontainer.denbound());
+					bool tmp = Givaro::reconstructRational(*num_p, tmp_den, zz_p_den, modulus, _lcontainer.numbound(), _lcontainer.denbound());
 #ifdef RSTIMING
 					++counter;
 #endif
@@ -1173,7 +1206,8 @@ namespace LinBox
 						}
 					}
 					else {
-						std::cout << "ERROR in reconstruction ? (ET)\n" << std::flush;
+						commentator().report()
+						<< "ERROR in reconstruction ? (ET)\n" << std::flush;
 					}
 
 				}
@@ -1189,7 +1223,7 @@ namespace LinBox
 #endif
 			Integer g;
 			Integer abs_num_p(0);
-			_r. init (g, 0);
+			_r. assign(g, _r.zero);
 			_r. gcdin (g, den);
 			for (num_p = num. begin(); num_p != num. end(); ++ num_p) {
 				_r. gcdin (g, *num_p);
@@ -1275,20 +1309,18 @@ namespace LinBox
 			std::cout<<"magnitude time:                 "<<magn<<"\n";
 
 			// some constants
-			Integer zero;
-			_r.init(zero,0);
-			Vector zero_digit(_lcontainer.size(),zero);
+			Vector zero_digit(_lcontainer.size(),_r.zero);
 
 			// store approximation as a polynomial and evaluate by baby step giant step
 			std::vector<Vector>  digit_approximation(length,zero_digit);
 
 			// store real approximation
-			Vector real_approximation(size,zero);
+			Vector real_approximation(size,_r.zero);
 			Vector last_real_approximation;
 
 			// store modulus (intially set to 1)
 			Integer modulus, last_modulus;
-			_r.init(modulus, 1);
+			_r.assign(modulus, _r.one);
 
 			typename LiftingContainer::const_iterator iter = _lcontainer.begin();
 
@@ -1298,11 +1330,11 @@ namespace LinBox
 			size_t endingsteps  =minsteps;
 
 			// switchers
-			bool latticeOK=false, numeratorOK=false, domoresteps=true, domorelattice=true;
+			bool numeratorOK=false, domoresteps=true, domorelattice=true;
 
 			// common denominator
 			Integer common_denom;
-			_r.init(common_denom,1);
+			_r.assign(common_denom,_r.one);
 
 			// bad numerator index
 			size_t bad_num_index=0;
@@ -1327,7 +1359,7 @@ namespace LinBox
 				if (domoresteps){
 
 					// compute the padic digits
-					for (size_t i = startingsteps ;  (i< endingsteps) && (iter.next(digit_approximation[i]));++i) {
+					for (size_t i = startingsteps ;  (i< endingsteps) && (iter.next(digit_approximation[(size_t)i]));++i) {
 						_r.mulin(modulus,prime);
 					}
 
@@ -1343,7 +1375,7 @@ namespace LinBox
 
 					if (startingsteps != 0){
 						for (size_t i=0;i<size;++i){
-							_r.axpyin(last_real_approximation[i],real_approximation[i], last_modulus);
+							_r.axpyin(last_real_approximation[(size_t)i],real_approximation[(size_t)i], last_modulus);
 						}
 						real_approximation=last_real_approximation;
 					}
@@ -1371,16 +1403,16 @@ namespace LinBox
 				// NTL::clear(Lattice);
 				// Lattice[0][0]=1;
 				// for (size_t i= bad_num_index+1;i< bad_num_index+k+1;++i){
-					// Lattice[i][i]=m;//not working when bad index <> 0
-					// _r.convert(tmp_int, real_approximation[i-1]);
+					// Lattice[(size_t)i][(size_t)i]=m;//not working when bad index <> 0
+					// _r.convert(tmp_int, real_approximation[(size_t)i-1]);
 					// tmp=NTL::to_ZZ((std::string(tmp_int)).c_str());
-					// Lattice[0][i]=tmp;//not working when bad index <> 0
+					// Lattice[0][(size_t)i]=tmp;//not working when bad index <> 0
 				// }
 				BlasMatrix<Ring> Lattice(_r,k+1,k+1);
 				Lattice.setEntry(0,0,_r.one);
 				for (size_t i= bad_num_index+1;i< bad_num_index+k+1;++i){
 					Lattice.setEntry(i,i,mod);//not working when bad index <> 0
-					_r.convert(tmp_int, real_approximation[i-1]);
+					_r.convert(tmp_int, real_approximation[(size_t)i-1]);
 					Lattice.setEntry(0,i,tmp_int);//not working when bad index <> 0
 				}
 
@@ -1400,8 +1432,8 @@ namespace LinBox
 
 
 				// check if the 1st row is the short vector
-				// Lattice[i][j] should work. Using standard getEntry though
-				latticeOK=true;
+				// Lattice[(size_t)i][j] should work. Using standard getEntry though
+				bool latticeOK=true;
 				tmp=abs(Lattice.getEntry(0,0))*ratio;
 				for (size_t i=1;i<k+1;++i){
 					for (size_t j=0;j<k+1;++j)
@@ -1431,7 +1463,7 @@ namespace LinBox
 					// integer dd= integer(0);
 					// for(long i = b - 1; i >= 0; --i) {
 						// dd *= base;
-						// dd += integer(byteArray[i]);
+						// dd += integer(byteArray[(size_t)i]);
 					// }
 					// delete [] byteArray;
 					myInteger dd = Lattice.getEntry(0,0);
@@ -1451,15 +1483,15 @@ namespace LinBox
 					numeratorOK=true;
 					// compute the numerators and check their validity according to the numerator  bound
 					for (size_t i=0;i<size;++i){
-						_r.mulin(real_approximation[i], denom);
-						_r.remin(real_approximation[i], modulus);
-						_r. sub (neg_approx, real_approximation[i], modulus);
+						_r.mulin(real_approximation[(size_t)i], denom);
+						_r.modin(real_approximation[(size_t)i], modulus);
+						_r. sub (neg_approx, real_approximation[(size_t)i], modulus);
 						_r. abs (abs_approx, neg_approx);
 
-						if ( _r.compare(real_approximation[i], numbound) < 0)
-							_r.assign(num[i], real_approximation[i]);
+						if ( _r.compare(real_approximation[(size_t)i], numbound) < 0)
+							_r.assign(num[(size_t)i], real_approximation[(size_t)i]);
 						else if (_r.compare(abs_approx, numbound) <0)
-							_r.assign(num[i], neg_approx);
+							_r.assign(num[(size_t)i], neg_approx);
 						else {
 							bad_num_index= std::min(i, size-k);
 							numeratorOK=false;
@@ -1483,7 +1515,7 @@ namespace LinBox
 						}
 
 						// check the validity of the solution according to n.||A||.||num||+ d.||b|| < modulus
-						integer check= size*MagnA*maxnum+dd*Magnb;
+						integer check= uint64_t(size)*MagnA*maxnum+dd*Magnb;
 
 						checksol.stop();
 						std::cout<<"checking solution time :        "<<checksol<<"\n\n";
@@ -1533,7 +1565,7 @@ namespace LinBox
 
 			if (neg_denom){
 				for (size_t i=0;i<size;++i)
-					_r.negin(num[i]);
+					_r.negin(num[(size_t)i]);
 			}
 #ifdef RSTIMING
 			tRecon.stop();
@@ -1618,20 +1650,18 @@ namespace LinBox
 			std::cout<<"magnitude time:                 "<<magn<<"\n";
 
 			// some constants
-			Integer zero;
-			_r.init(zero,0);
-			Vector zero_digit(_lcontainer.size(),zero);
+			Vector zero_digit(_lcontainer.size(),_r.zero);
 
 			// store approximation as a polynomial and evaluate by baby step giant step
 			std::vector<Vector>  digit_approximation(length,zero_digit);
 
 			// store real approximation
-			Vector real_approximation(size,zero);
+			Vector real_approximation(size,_r.zero);
 			Vector last_real_approximation;
 
 			// store modulus (intially set to 1)
 			Integer modulus, last_modulus;
-			_r.init(modulus, 1);
+			_r.assign(modulus, _r.one);
 
 			typename LiftingContainer::const_iterator iter = _lcontainer.begin();
 
@@ -1641,11 +1671,11 @@ namespace LinBox
 			size_t endingsteps  =minsteps;
 
 			// switchers
-			bool latticeOK=false, numeratorOK=false, domoresteps=true, domorelattice=true;
+			bool numeratorOK=false, domoresteps=true, domorelattice=true;
 
 			// common denominator
 			Integer common_denom;
-			_r.init(common_denom,1);
+			_r.assign(common_denom,_r.one);
 
 			// bad numerator index
 			size_t bad_num_index=0;
@@ -1672,7 +1702,7 @@ namespace LinBox
 					linbox_check(startingsteps != endingsteps);
 
 					// compute the padic digits
-					for (size_t i = startingsteps ;  (i< endingsteps) && (iter.next(digit_approximation[i]));++i) {
+					for (size_t i = startingsteps ;  (i< endingsteps) && (iter.next(digit_approximation[(size_t)i]));++i) {
 						_r.mulin(modulus,prime);
 					}
 
@@ -1688,7 +1718,7 @@ namespace LinBox
 
 					if (startingsteps != 0){
 						for (size_t i=0;i<size;++i){
-							_r.axpyin(last_real_approximation[i],real_approximation[i], last_modulus);
+							_r.axpyin(last_real_approximation[(size_t)i],real_approximation[(size_t)i], last_modulus);
 						}
 						real_approximation=last_real_approximation;
 					}
@@ -1711,9 +1741,9 @@ namespace LinBox
 				// ZZ_mat<mpz_t> Lattice(k+1,k+1) ;
 				// Lattice= new mpz_t*[k+2];
 				// for (size_t i=0;i<k+2;++i){
-					// Lattice[i]= new mpz_t[k+2];
+					// Lattice[(size_t)i]= new mpz_t[k+2];
 					// for (size_t j=0;j<k+2;++j)
-						// mpz_init(Lattice[i][j]);
+						// mpz_init(Lattice[(size_t)i][j]);
 				// }
 				BlasMatrix<Ring> Lattice(_r,k+1,k+1);
 
@@ -1724,11 +1754,11 @@ namespace LinBox
 
 				// Lattice.Set(0,0,Z_NR<mpz_t>(tmp.get_mpz()));
 				// for (size_t i=1;i< k+1;++i){
-					// mpz_set(Lattice[i][i],mod.get_mpz());
+					// mpz_set(Lattice[(size_t)i][(size_t)i],mod.get_mpz());
 					// Lattice.Set(i,i, Z_NR<mpz_t>(mod.get_mpz()) );
 					// _r.convert(tmp, real_approximation[bad_num_index+i-1]);
 					// Lattice.Set(0,i,Z_NR<mpz_t>(tmp.get_mpz()));
-					// mpz_set(Lattice[1][i],tmp.get_mpz());
+					// mpz_set(Lattice[1][(size_t)i],tmp.get_mpz());
 				// }
 				Lattice.setEntry(0,0,tmp);
 				for (size_t i=1;i< k+1;++i){
@@ -1754,7 +1784,7 @@ namespace LinBox
 
 
 				// check if the 1st row is the short vector
-				latticeOK=true;
+				bool latticeOK=true;
 				_r.mul(tmp, Lattice.getEntry(0,0),ratio);
 				// mpz_mul(tmp.get_mpz(), Lattice(0,0).GetData(),ratio.get_mpz());
 				for (size_t i=1;i<k+1;++i){
@@ -1773,8 +1803,8 @@ namespace LinBox
 				//delete the lattice
 				// for (size_t i=0;i<k+2;++i){
 					// for (size_t j=0;j<k+2;++j)
-						// mpz_clear(Lattice[i][j]);
-					// delete[] Lattice[i];
+						// mpz_clear(Lattice[(size_t)i][j]);
+					// delete[] Lattice[(size_t)i];
 				// }
 				// delete[] Lattice;
 
@@ -1801,15 +1831,15 @@ namespace LinBox
 					numeratorOK=true;
 					// compute the numerators and check their validity according to the numerator  bound
 					for (size_t i=0;i<size;++i){
-						_r.mulin(real_approximation[i], common_denom);
-						_r.remin(real_approximation[i], modulus);
-						_r. sub (neg_approx, real_approximation[i], modulus);
+						_r.mulin(real_approximation[(size_t)i], common_denom);
+						_r.modin(real_approximation[(size_t)i], modulus);
+						_r. sub (neg_approx, real_approximation[(size_t)i], modulus);
 						_r. abs (abs_approx, neg_approx);
 
-						if ( _r.compare(real_approximation[i], numbound) < 0)
-							_r.assign(num[i], real_approximation[i]);
+						if ( _r.compare(real_approximation[(size_t)i], numbound) < 0)
+							_r.assign(num[(size_t)i], real_approximation[(size_t)i]);
 						else if (_r.compare(abs_approx, numbound) <0)
-							_r.assign(num[i], neg_approx);
+							_r.assign(num[(size_t)i], neg_approx);
 						else {
 							bad_num_index= std::min(i, size-k);
 							numeratorOK=false;
@@ -1833,7 +1863,7 @@ namespace LinBox
 						}
 
 						// check the validity of the solution according to n.||A||.||num||+ d.||b|| < modulus
-						integer check= size*MagnA*maxnum+dd*Magnb;
+						integer check= uint64_t(size)*MagnA*maxnum+dd*Magnb;
 
 						checksol.stop();
 						std::cout<<"checking solution time :        "<<checksol<<"\n\n";
@@ -1883,7 +1913,7 @@ namespace LinBox
 
 			if (neg_denom){
 				for (size_t i=0;i<size;++i)
-					_r.negin(num[i]);
+					_r.negin(num[(size_t)i]);
 			}
 #ifdef RSTIMING
 			tRecon.stop();
@@ -1960,20 +1990,18 @@ namespace LinBox
 			std::cout<<"magnitude time:                 "<<magn<<"\n";
 
 			// some constants
-			Integer zero;
-			_r.init(zero,0);
-			Vector zero_digit(_lcontainer.size(),zero);
+			Vector zero_digit(_lcontainer.size(),_r.zero);
 
 			// store approximation as a polynomial and evaluate by baby step giant step
 			std::vector<Vector>  digit_approximation(length,zero_digit);
 
 			// store real approximation
-			Vector real_approximation(size,zero);
+			Vector real_approximation(size,_r.zero);
 			Vector last_real_approximation;
 
 			// store modulus (intially set to 1)
 			Integer modulus, last_modulus;
-			_r.init(modulus, 1);
+			_r.assign(modulus, _r.one);
 
 			typename LiftingContainer::const_iterator iter = _lcontainer.begin();
 
@@ -1983,11 +2011,11 @@ namespace LinBox
 			size_t endingsteps  =minsteps;
 
 			// switchers
-			bool latticeOK=false, numeratorOK=false, domoresteps=true, domorelattice=true;
+			bool numeratorOK=false, domoresteps=true, domorelattice=true;
 
 			// common denominator
 			Integer common_denom;
-			_r.init(common_denom,1);
+			_r.assign(common_denom,_r.one);
 
 			// bad numerator index
 			size_t bad_num_index=0;
@@ -2014,7 +2042,7 @@ namespace LinBox
 					linbox_check(startingsteps != endingsteps);
 
 					// compute the padic digits
-					for (size_t i = startingsteps ;  (i< endingsteps) && (iter.next(digit_approximation[i]));++i) {
+					for (size_t i = startingsteps ;  (i< endingsteps) && (iter.next(digit_approximation[(size_t)i]));++i) {
 						_r.mulin(modulus,prime);
 					}
 
@@ -2030,7 +2058,7 @@ namespace LinBox
 
 					if (startingsteps != 0){
 						for (size_t i=0;i<size;++i){
-							_r.axpyin(last_real_approximation[i],real_approximation[i], last_modulus);
+							_r.axpyin(last_real_approximation[(size_t)i],real_approximation[(size_t)i], last_modulus);
 						}
 						real_approximation=last_real_approximation;
 					}
@@ -2056,9 +2084,9 @@ namespace LinBox
 
 				Lattice[0]=tmp;
 				for (size_t i=1;i< k+1;++i){
-					Lattice[i*(k+1)+i]=mod;
+					Lattice[(size_t)i*(k+1)+i]=mod;
 					_r.convert(tmp, real_approximation[bad_num_index+i-1]);
-					Lattice[i]=tmp;
+					Lattice[(size_t)i]=tmp;
 				}
 
 				// ratio to check the validity of the denominator compare to the entries in the reduced lattice
@@ -2075,11 +2103,11 @@ namespace LinBox
 
 
 				// check if the 1st row is the short vector
-				latticeOK=true;
+				bool latticeOK=true;
 				tmp = Givaro::abs(L3[0][0]*ratio);
 				for (size_t i=1;i<k+1;++i){
 					for (size_t j=0;j<k+1;++j)
-						if (tmp > Givaro::abs(L3[i][j])){
+						if (tmp > Givaro::abs(L3[(size_t)i][j])){
 							latticeOK=false;
 							break;
 						}
@@ -2112,15 +2140,15 @@ namespace LinBox
 					numeratorOK=true;
 					// compute the numerators and check their validity according to the numerator  bound
 					for (size_t i=0;i<size;++i){
-						_r.mulin(real_approximation[i], common_denom);
-						_r.remin(real_approximation[i], modulus);
-						_r. sub (neg_approx, real_approximation[i], modulus);
+						_r.mulin(real_approximation[(size_t)i], common_denom);
+						_r.modin(real_approximation[(size_t)i], modulus);
+						_r. sub (neg_approx, real_approximation[(size_t)i], modulus);
 						_r. abs (abs_approx, neg_approx);
 
-						if ( _r.compare(real_approximation[i], numbound) < 0)
-							_r.assign(num[i], real_approximation[i]);
+						if ( _r.compare(real_approximation[(size_t)i], numbound) < 0)
+							_r.assign(num[(size_t)i], real_approximation[(size_t)i]);
 						else if (_r.compare(abs_approx, numbound) <0)
-							_r.assign(num[i], neg_approx);
+							_r.assign(num[(size_t)i], neg_approx);
 						else {
 							bad_num_index= std::min(i, size-k);
 							numeratorOK=false;
@@ -2144,7 +2172,7 @@ namespace LinBox
 						}
 
 						// check the validity of the solution according to n.||A||.||num||+ d.||b|| < modulus
-						integer check= size*MagnA*maxnum+dd*Magnb;
+						integer check= uint64_t(size)*MagnA*maxnum+dd*Magnb;
 
 						checksol.stop();
 						std::cout<<"checking solution time :        "<<checksol<<"\n\n";
@@ -2194,7 +2222,7 @@ namespace LinBox
 
 			if (neg_denom){
 				for (size_t i=0;i<size;++i)
-					_r.negin(num[i]);
+					_r.negin(num[(size_t)i]);
 			}
 #ifdef RSTIMING
 			tRecon.stop();
